/*
 * infoproviders.js
 *
 *  Created on: 08.02.2016
 *      Author: Holger Dahm
 *
 * the module provides all infoprovider embracing stuff.
 *
 * Note: add all infoproviders necessary to service your app
 */

// initialize available infoproviders
var weatherinfoprovider = require("./weatherinfoprovider");

// used for any standard initialization
var infoproviders = [ weatherinfoprovider ];



/*
 * standard initialization of infoproviders
 */

// (default) options used in this infoprovider
// as well as all info providers
// Note: call setOptions() before doing anything!
var options;

function setOptions(values) {
	options = values;

	infoproviders.forEach(function(infoProvider, index, array) {
		infoProvider.setOptions(options);
	});
}

// app controller used in this module
// as well as all info providers for
// json or xml request, caching as well as message to pebble send operations
// Note: call setAppController() before doing anything!
var appcontroller;

function setAppController(controller) {
	appcontroller = controller;

	infoproviders.forEach(function(infoProvider, index, array) {
		infoProvider.setAppController(appcontroller);
	});
}



/*
 * push still cached infos to appmessage queue
 * means, test if "old" information is available; in case send it to pebble
 */

function pushCachedInfosToAppmessageQueue() {
	appcontroller.pushLocalStorageToAppMessages(weatherinfoprovider.WEATHER_ID);
}

/*
 * clear all cached infos
 */

function clearCache() {
    localStorage.removeItem(weatherinfoprovider.WEATHER_ID);
}

/*
 * central app processing in reaction to pebble request
 */

function process(selectionID) {
    try {
        // start processing; flag is unset in sendAppMessage if last element of queue is send
        appcontroller.setProcessing(true);

        switch (selectionID) {
            case 0:
            	weatherinfoprovider.fetch(0);
                break;
            default:
                console.log("Error - received selection " + selectionID + " is not defined.");
            	appcontroller.setProcessing(false);
                break;
        }
    }
    catch (error) {
        console.log("Error during processing!\n" + error.stack);
        appcontroller.appMessageQueue.push({
            "message" : {
                "nodata" : "processing error"
            }
        });
        appcontroller.sendAppMessage(true); // unsets processing = true ;)
    }
}

function processPebbleRequest(event) {
	// test if processing is still ongoing or request is really requested
    var selectionID = event.payload.selectionID;
    console.log("Got data request! [selectionID: " + selectionID +  "]");

    if (appcontroller.isProcessing()) {
        // nothing remains to do
        console.log("Stopped because I'm still processing!");
        return;
    }

    process(selectionID);
}



/*
 * transparent infoprovider interface
 */

module.exports = {
		// set (default) options used in this module
		// as well as all info providers
		setOptions : setOptions,

		// set app controller used in this module
		// as well as all info providers
		setAppController : setAppController,

		// push still cached infos to appmessage queue
		pushCachedInfosToAppmessageQueue : pushCachedInfosToAppmessageQueue,

		// clear all cached infos
		clearCache : clearCache,

		// process pebble request
		processPebbleRequest : processPebbleRequest
};
