/*
 * pebblehelper.js
 *
 * the module provides a couple of useful helper methods
 */


// flag that indicates if platform changes since last JS call
var isNewPlatform = false;

/*
 * detect current pebble platform functions
 */

function getPebblePlatform() {
	if(Pebble.getActiveWatchInfo) {
	  var watchinfo= Pebble.getActiveWatchInfo();
	  	return watchinfo.platform;
	} else {
	    return "aplite";
	}
}

function isAplite() {
	return getPebblePlatform() == "aplite";
}

function isBasalt() {
	return getPebblePlatform() == "basalt";
}

function isChalk() {
	return getPebblePlatform() == "chalk";
}

function isDiorite() {
	return getPebblePlatform() == "diorite";
}

function isEmery() {
	return getPebblePlatform() == "emery";
}


function detectPebblePlatformChange() {
	var oldPlatform = localStorage.getItem("platform");
	var newPlatform = getPebblePlatform();

	isNewPlatform = (oldPlatform != newPlatform);

	localStorage.setItem("platform", newPlatform);

	if (isNewPlatform) {
		console.log("Detetcted pebble platform change from '" + oldPlatform + "' to '" + newPlatform + "'.");
	}
}

function isNewPebblePlatform() {
	return isNewPlatform;
}

/*
 * special date functions
 */

function isWeakday() {
    var day = new Date().getDay();
    if (day > 0 && day < 6) {
        //console.log("Today is a weekday.");
        return true;
    }

    return false;
}

function nowIsBetween(min, max){
    var now = new Date();
    var minDate = new Date((now.getMonth() + 1) + " " + now.getDate() + ", " + now.getFullYear() + " " + min + ":00");
    var maxDate = new Date((now.getMonth() + 1) + " " + now.getDate() + ", " + now.getFullYear() + " " + max + ":00");

    if(now.getTime() < minDate.getTime() || now.getTime() > maxDate.getTime()) {
        //console.log("Actual time is not between " + min + " and " + max);
        return false;
    }

    return true;
}

/*
 * special array functions
 */

/**
 * Overwrites obj1's values with obj2's and adds obj2's if non existent in obj1
 * @param obj1
 * @param obj2
 * @returns obj3 a new object based on obj1 and obj2
 */
function mergeArrays(obj1,obj2){
    var obj3 = {};

    for (var key in obj1) {
        if ( key === 'length' || !obj1.hasOwnProperty(key) ) {
            continue;
        }
        obj3[key] = obj1[key];
    }
    for (var key in obj2) {
        if ( key === 'length' || !obj2.hasOwnProperty(key) ) {
            continue;
        }
        obj3[key] = obj2[key];
    }
    return obj3;
}

function isArray(value) {
    return Object.prototype.toString.call(value) === '[object Array]';
}

function getValuesByKeyName(object, key) {
    var objects = [];

    for (var i in object) {
        if (!object.hasOwnProperty(i)) continue;

        if (i == key) {
            objects.push(object[i]);
        }
        else if (isArray(object[i])) {
            object[i].forEach(function(element, index, array) {
                objects = objects.concat(getValuesByKeyName(element, key));
            });
        }
        else if (typeof object[i] == 'object') {
            objects = objects.concat(getValuesByKeyName(object[i], key));
        }
    }

    return objects;
}

/*
 * functions to translate special HTML chars to commonly used ASCII chars
 */

function get_html_translation_table(table, quote_style) {
    var entities = {}, hash_map = {};
    var constMappingTable = {}, constMappingQuoteStyle = {};
    var useTable = {}, useQuoteStyle = {};

    // Translate arguments
    constMappingTable[0]      = 'HTML_SPECIALCHARS';
    constMappingTable[1]      = 'HTML_ENTITIES';
    constMappingQuoteStyle[0] = 'ENT_NOQUOTES';
    constMappingQuoteStyle[2] = 'ENT_COMPAT';
    constMappingQuoteStyle[3] = 'ENT_QUOTES';

    useTable = !isNaN(table) ? constMappingTable[table] : table ? table.toUpperCase() : 'HTML_SPECIALCHARS';
    useQuoteStyle = !isNaN(quote_style) ? constMappingQuoteStyle[quote_style] : quote_style ? quote_style.toUpperCase() : 'ENT_COMPAT';

    if (useTable !== 'HTML_SPECIALCHARS' && useTable !== 'HTML_ENTITIES') {
        throw new Error('Table: ' + useTable + ' not supported');
        // return false;
    }

    entities['38'] = '&amp;';
    if (useTable === 'HTML_ENTITIES') {
        entities['160'] = '&nbsp;';
        entities['161'] = '&iexcl;';
        entities['162'] = '&cent;';
        entities['163'] = '&pound;';
        entities['164'] = '&curren;';
        entities['165'] = '&yen;';
        entities['166'] = '&brvbar;';
        entities['167'] = '&sect;';
        entities['168'] = '&uml;';
        entities['169'] = '&copy;';
        entities['170'] = '&ordf;';
        entities['171'] = '&laquo;';
        entities['172'] = '&not;';
        entities['173'] = '&shy;';
        entities['174'] = '&reg;';
        entities['175'] = '&macr;';
        entities['176'] = '&deg;';
        entities['177'] = '&plusmn;';
        entities['178'] = '&sup2;';
        entities['179'] = '&sup3;';
        entities['180'] = '&acute;';
        entities['181'] = '&micro;';
        entities['182'] = '&para;';
        entities['183'] = '&middot;';
        entities['184'] = '&cedil;';
        entities['185'] = '&sup1;';
        entities['186'] = '&ordm;';
        entities['187'] = '&raquo;';
        entities['188'] = '&frac14;';
        entities['189'] = '&frac12;';
        entities['190'] = '&frac34;';
        entities['191'] = '&iquest;';
        entities['192'] = '&Agrave;';
        entities['193'] = '&Aacute;';
        entities['194'] = '&Acirc;';
        entities['195'] = '&Atilde;';
        entities['196'] = '&Auml;';
        entities['197'] = '&Aring;';
        entities['198'] = '&AElig;';
        entities['199'] = '&Ccedil;';
        entities['200'] = '&Egrave;';
        entities['201'] = '&Eacute;';
        entities['202'] = '&Ecirc;';
        entities['203'] = '&Euml;';
        entities['204'] = '&Igrave;';
        entities['205'] = '&Iacute;';
        entities['206'] = '&Icirc;';
        entities['207'] = '&Iuml;';
        entities['208'] = '&ETH;';
        entities['209'] = '&Ntilde;';
        entities['210'] = '&Ograve;';
        entities['211'] = '&Oacute;';
        entities['212'] = '&Ocirc;';
        entities['213'] = '&Otilde;';
        entities['214'] = '&Ouml;';
        entities['215'] = '&times;';
        entities['216'] = '&Oslash;';
        entities['217'] = '&Ugrave;';
        entities['218'] = '&Uacute;';
        entities['219'] = '&Ucirc;';
        entities['220'] = '&Uuml;';
        entities['221'] = '&Yacute;';
        entities['222'] = '&THORN;';
        entities['223'] = '&szlig;';
        entities['224'] = '&agrave;';
        entities['225'] = '&aacute;';
        entities['226'] = '&acirc;';
        entities['227'] = '&atilde;';
        entities['228'] = '&auml;';
        entities['229'] = '&aring;';
        entities['230'] = '&aelig;';
        entities['231'] = '&ccedil;';
        entities['232'] = '&egrave;';
        entities['233'] = '&eacute;';
        entities['234'] = '&ecirc;';
        entities['235'] = '&euml;';
        entities['236'] = '&igrave;';
        entities['237'] = '&iacute;';
        entities['238'] = '&icirc;';
        entities['239'] = '&iuml;';
        entities['240'] = '&eth;';
        entities['241'] = '&ntilde;';
        entities['242'] = '&ograve;';
        entities['243'] = '&oacute;';
        entities['244'] = '&ocirc;';
        entities['245'] = '&otilde;';
        entities['246'] = '&ouml;';
        entities['247'] = '&divide;';
        entities['248'] = '&oslash;';
        entities['249'] = '&ugrave;';
        entities['250'] = '&uacute;';
        entities['251'] = '&ucirc;';
        entities['252'] = '&uuml;';
        entities['253'] = '&yacute;';
        entities['254'] = '&thorn;';
        entities['255'] = '&yuml;';
    }

    if (useQuoteStyle !== 'ENT_NOQUOTES') {
        entities['34'] = '&quot;';
    }
    if (useQuoteStyle === 'ENT_QUOTES') {
        entities['39'] = '&#39;';
    }
    entities['60'] = '&lt;';
    entities['62'] = '&gt;';

    // ascii decimals to real symbols
    for (var decimal in entities) {
        if (entities.hasOwnProperty(decimal)) {
            hash_map[String.fromCharCode(decimal)] = entities[decimal];
        }
    }

    return hash_map;
}

function decodeHTML(string, quote_style) {
    // discuss at: http://phpjs.org/functions/html_entity_decode/
    // original by: john (http://www.jd-tech.net)
    var hash_map = {}, symbol = '', tmp_str = '', entity = '';
    tmp_str = string.toString();

    // decode unicode characters
    var r = /\\u([\d\w]{4})/gi;
    tmp_str = tmp_str.replace(r, function(match, grp) {
        return String.fromCharCode(parseInt(grp, 16));
    });
    tmp_str = unescape(tmp_str);
    // remove all kinds of HTML tags and all kinds of line breaks
    tmp_str = tmp_str.replace(/(<([^>]+)>)/ig, "")
                     .replace(/(\r\n|\n|\r)/gm,"")
                     .replace(/&rsquo;/gm,"'")
                     .replace(/&#58;/gm,":")
                     .replace(/&#39;/gm,"'");

    // decode html entities
    if (false === (hash_map = get_html_translation_table('HTML_ENTITIES', quote_style))) {
        return false;
    }

    // fix &amp; problem
    // http://phpjs.org/functions/get_html_translation_table:416#comment_97660
    delete (hash_map['&']);
    hash_map['&'] = '&amp;';

    for (symbol in hash_map) {
        entity = hash_map[symbol];
        tmp_str = tmp_str.split(entity).join(symbol);
    }
    tmp_str = tmp_str.split('&#039;').join("'");

    return tmp_str;
}

/*
 * special string functions
 */

function getByteLength(value) {
    // Force string type
    value = String(value);

    var byteLen = 0;
    for (var i = 0; i < value.length; i++) {
        var c = value.charCodeAt(i);

        var bytes = c < (1 <<  7) ? 1 :
                    c < (1 << 11) ? 2 :
                    c < (1 << 16) ? 3 :
                    c < (1 << 21) ? 4 :
                    c < (1 << 26) ? 5 :
                    c < (1 << 31) ? 6 : Number.NaN;
        //console.log(i + ": '" + value.charAt(i) + "' char code '" + c + "' has " + bytes + " bytes");

        byteLen += bytes;
    }
    return byteLen;
}

function getDecodedTrimedString(string) {
	return decodeHTML(string).replace(/\|/g,' ').replace(/^\s+|\s+$/g,'');
}



/*
 * module interace
 */

module.exports = {
	// detect current pebble platform functions
	getPebblePlatform : getPebblePlatform,
	isAplite  : isAplite,
	isBasalt  : isBasalt,
	isChalk   : isChalk,
	isDiorite : isDiorite,
	isEmery   : isEmery,

	detectPebblePlatformChange : detectPebblePlatformChange,
	isNewPebblePlatform : isNewPebblePlatform,

	// special date functions
	isWeakday : isWeakday,
	nowIsBetween : nowIsBetween,

	// special array functions
	mergeArrays : mergeArrays,
	isArray : isArray,
	getValuesByKeyName : getValuesByKeyName,

	// functions to translate special HTML chars to commonly used ASCII chars
	decodeHTML : decodeHTML,

	// special string functions
	getByteLength : getByteLength,
	getDecodedTrimedString : getDecodedTrimedString
};
